<?php

namespace SenSai;

use FML\ManiaLink;
use FML\Controls\Frame;
use FML\Controls\Quad;
use FML\Controls\Labels\Label_Text;
use FML\Controls\Control;

use ManiaControl\Callbacks\Structures\ShootMania\OnArmorEmptyStructure;
use ManiaControl\Callbacks\Structures\ShootMania\OnEliteEndTurnStructure;
use ManiaControl\Callbacks\Structures\ShootMania\OnEliteStartTurnStructure;
use ManiaControl\Logger;
use ManiaControl\ManiaControl;
use ManiaControl\Callbacks\Callbacks;
use ManiaControl\Callbacks\CallbackListener;
use ManiaControl\Callbacks\CallbackManager;
use ManiaControl\Callbacks\TimerListener;
use ManiaControl\Callbacks\TimerManager;
use ManiaControl\Plugins\Plugin;
use ManiaControl\Maps\MapManager;
use ManiaControl\Players\Player;
use ManiaControl\Players\PlayerManager;
use ManiaControl\Manialinks\ManialinkPageAnswerListener;
use ManiaControl\Manialinks\IconManager;
use ManiaControl\Manialinks\ManialinkManager;
use ManiaControl\Commands\CommandListener;

/**
 * Elite Extra Plugin
 *
 * @author Sen-Sai
 */

class EliteX implements CallbackListener, TimerListener, ManialinkPageAnswerListener, CommandListener, Plugin {
    /**
     * Constants
     */
    const PLUGIN_ID			= 68;
    const PLUGIN_VERSION	= 1.4;
    const PLUGIN_NAME		= 'Elite Extra';
    const PLUGIN_AUTHOR		= 'G]R [SG-1]Sen-sai';

    // Loadscreen properties

    const MLID_WIDGET				= 'EliteX.ID';
    const SETTING_ELITEX_LOGO		= 'Logo URL';
	const SETTING_ELITEX_BG_R		= 'Background Red (0-255)';
	const SETTING_ELITEX_BG_G		= 'Background Green (0-255)';
	const SETTING_ELITEX_BG_B		= 'Background Blue (0-255)';
	const SETTING_ELITEX_BG_A		= 'Background Alpha (0-100)';
	const ELITEX_IMG_RIP			= 'rip.png';
	const ELITEX_IMG_MIN			= 'min.png';
	const ELITEX_IMG_MAX			= 'max.png';
	const ELITEX_IMG_ECH0			= 'echelon0.png';
	const ELITEX_IMG_ECH1			= 'echelon1.png';
	const ELITEX_IMG_ECH2			= 'echelon2.png';
	const ELITEX_IMG_ECH3			= 'echelon3.png';
	const ELITEX_IMG_ECH4			= 'echelon4.png';
	const ELITEX_IMG_ECH5			= 'echelon5.png';
	const ELITEX_IMG_ECH6			= 'echelon6.png';
	const ELITEX_IMG_ECH7			= 'echelon7.png';
	const ELITEX_IMG_ECH8			= 'echelon8.png';
	const ELITEX_IMG_ECH9			= 'echelon9.png';
	const ACTION_MINIMIZE_WIDGET	= 'EliteX.Minimize';
	const ACTION_MAXIMIZE_WIDGET  	= 'EliteX.Maximize';

	private $img = array();
	private $teams = array();
	private $playing = false;
	private $widget = array();

    /**
     * Private Properties
     */
    /**
     * @var maniaControl $maniaControl
     */
    private $maniaControl = null;


    /**
     * Prepares the Plugin
     *
     * @param ManiaControl $maniaControl
     * @return mixed
     */
    public static function prepare(ManiaControl $maniaControl) {
        //do nothing
    }

    /**
     * Load the plugin
     *
     * @param ManiaControl $maniaControl
     * @return bool
     */
    public function load(ManiaControl $maniaControl) {
        $this->maniaControl = $maniaControl;
		if ($this->maniaControl->getServer()->titleId != 'SMStormElite@nadeolabs') {
			$error = 'This plugin only supports the Elite Mode (check Settings)!';
			throw new \Exception($error);
		}

        //Register callbacks

        $this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_ELITEX_LOGO, 'http://files.designburo.nl/shootmania/images/glrimratten.png');
		$this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_ELITEX_BG_R, '0');
		$this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_ELITEX_BG_G, '0');
		$this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_ELITEX_BG_B, '0');
		$this->maniaControl->getSettingManager()->initSetting($this, self::SETTING_ELITEX_BG_A, '50');

		$file = $this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_ELITEX_LOGO);
		$array = get_headers($file);
		$string = $array[0];
		if(!strpos($string,"200")) {

			$this->maniaControl->getChat()->sendInformation('The URL does not exist. Please check. Elite Extra Plugin disabled.', null, false);
			return;
		}
		// new callbacks
	    $this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::SM_ELITE_STARTTURN, $this, 'handleCallbackEliteStartTurn');
	    $this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::SM_ELITE_ENDTURN, $this, 'handleCallbackEliteEndTurn');
	    $this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::ENDMAP, $this, 'handleCallbackEndMap');
	    $this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::MP_WARMUP_START, $this, 'handleCallbackBeginWarmup');
	    $this->maniaControl->getCallbackManager()->registerCallbackListener(Callbacks::SM_ONARMOREMPTY, $this, 'handleCallbackOnArmorEmpty');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(PlayerManager::CB_PLAYERINFOCHANGED, $this, 'handlePlayerInfoChangedCallback');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(PlayerManager::CB_PLAYERCONNECT, $this, 'handlePlayerConnectCallback');
		$this->maniaControl->getCallbackManager()->registerCallbackListener(PlayerManager::CB_PLAYERDISCONNECT, $this, 'updatePlayerLeft');
		$this->maniaControl->getCommandManager()->registerCommandListener('hidewidget', $this, 'hide', true);
		$this->maniaControl->getCommandManager()->registerCommandListener('showwidget', $this, 'show', true);

		$ext = substr(strrchr($file, "."), 1);
		if ($ext=="jpg" || $ext=="png")
		{


			$this->maniaControl->getManialinkManager()->getIconManager()->addIcon(self::ELITEX_IMG_RIP,'http://files.designburo.nl/shootmania/images/');
			$this->maniaControl->getManialinkManager()->getIconManager()->addIcon(self::ELITEX_IMG_MIN,'http://files.designburo.nl/shootmania/images/');
			$this->maniaControl->getManialinkManager()->getIconManager()->addIcon(self::ELITEX_IMG_MAX,'http://files.designburo.nl/shootmania/images/');
			$this->maniaControl->getManialinkManager()->getIconManager()->addIcon(self::ELITEX_IMG_ECH0,'http://files.designburo.nl/shootmania/images/');
			$this->maniaControl->getManialinkManager()->getIconManager()->addIcon(self::ELITEX_IMG_ECH1,'http://files.designburo.nl/shootmania/images/');
			$this->maniaControl->getManialinkManager()->getIconManager()->addIcon(self::ELITEX_IMG_ECH2,'http://files.designburo.nl/shootmania/images/');
			$this->maniaControl->getManialinkManager()->getIconManager()->addIcon(self::ELITEX_IMG_ECH3,'http://files.designburo.nl/shootmania/images/');
			$this->maniaControl->getManialinkManager()->getIconManager()->addIcon(self::ELITEX_IMG_ECH4,'http://files.designburo.nl/shootmania/images/');
			$this->maniaControl->getManialinkManager()->getIconManager()->addIcon(self::ELITEX_IMG_ECH5,'http://files.designburo.nl/shootmania/images/');
			$this->maniaControl->getManialinkManager()->getIconManager()->addIcon(self::ELITEX_IMG_ECH6,'http://files.designburo.nl/shootmania/images/');
			$this->maniaControl->getManialinkManager()->getIconManager()->addIcon(self::ELITEX_IMG_ECH7,'http://files.designburo.nl/shootmania/images/');
			$this->maniaControl->getManialinkManager()->getIconManager()->addIcon(self::ELITEX_IMG_ECH8,'http://files.designburo.nl/shootmania/images/');
			$this->maniaControl->getManialinkManager()->getIconManager()->addIcon(self::ELITEX_IMG_ECH9,'http://files.designburo.nl/shootmania/images/');
			$this->img['logo']=$file;
			$this->maniaControl->getManialinkManager()->getIconManager()->preloadIcons();

		}
		else
		{
			$this->maniaControl->getChat()->sendInformation('The URL must be either .png or .jpg', null, false);
			$this->maniaControl->getCallbackManager()->unregisterCallbackListener($this);
			return;
		}

		$this->teams['defender'][0]["status"]="empty";
		$this->teams['defender'][1]["status"]="empty";
		$this->teams['defender'][2]["status"]="empty";
		$this->teams['attacker']="";
    }


	public function handlePlayerConnectCallback(Player $player)
	{
		if($this->playing==false) return;
		$this->displayTurnWidget($player->login);

	}


	public function handlePlayerInfoChangedCallback(Player $player)
	{
		if($this->playing==false) return;
		$login = $player->login;
		if ($this->teams['defender'][0]["status"]=="playing")
		{
			if($login==$this->teams['defender'][0]["login"])
			{
				$this->teams['defender'][0]["status"]="afk";
				$this->widgetChoice();
			}
		}

		if ($this->teams['defender'][1]["status"]=="playing")
		{
			if($login==$this->teams['defender'][1]["login"])
			{
				$this->teams['defender'][1]["status"]="afk";
				$this->widgetChoice();
			}
		}

		if ($this->teams['defender'][2]["status"]=="playing")
		{
			if($login==$this->teams['defender'][2]["login"])
			{
				$this->teams['defender'][2]["status"]="afk";
				$this->widgetChoice();
			}
		}


	}

	public function updatePlayerLeft(Player $player)
	{
		if($this->playing==false) return;
		$login = $player->login;
		$this->widget[$login]="left";
		if ($this->teams['defender'][0]["status"]!=="empty")
		{
			if($login==$this->teams['defender'][0]["login"])
			{
				$this->teams['defender'][0]["status"]="left";
				$this->widgetChoice();
			}
		}
		if ($this->teams['defender'][1]["status"]!=="empty")
		{
			if($login==$this->teams['defender'][1]["login"])
			{
				$this->teams['defender'][1]["status"]="left";
				$this->widgetChoice();
			}
		}
		if ($this->teams['defender'][2]["status"]!=="empty")
		{
			if($login==$this->teams['defender'][2]["login"])
			{
				$this->teams['defender'][2]["status"]="left";
				$this->widgetChoice();
			}
		}

	}

	public function updateKill($info)
	{
		if($this->playing==false) return;
		$victim = $info["victim"];
		if ($this->teams['defender'][0]["status"]=="playing")
		{
			if($victim==$this->teams['defender'][0]["login"])
			{
				$this->teams['defender'][0]["status"]="dead";
				$this->widgetChoice();
			}
		}
		if ($this->teams['defender'][1]["status"]=="playing")
		{
			if($victim==$this->teams['defender'][1]["login"])
			{
				$this->teams['defender'][1]["status"]="dead";
				$this->widgetChoice();
			}
		}
		if ($this->teams['defender'][2]["status"]=="playing")
		{
			if($victim==$this->teams['defender'][2]["login"])
			{
				$this->teams['defender'][2]["status"]="dead";
				$this->widgetChoice();
			}
		}
	}

	public function getEchelon($rank)
	{
		if($rank<=9) {
			$ret = "echelon0.png";
			return $ret;
		}
		$echelon = floor($rank/10000);
		$ret = "echelon".strval($echelon).".png";
		return $ret;

	}

	public function widgetChoice() {
		$this->displayTurnWidget();
	}

	public function displayTurnWidget($to=false){

		/*	const ACTION_MINIMIZE_WIDGET	= 'EliteX.Minimize';
			const ACTION_MAXIMIZE_WIDGET  	= 'EliteX.Maximize';
		*/
		//if ($action !== self::ACTION_SOUNDBOXPLAY) return;
		//setAction(self::ACTION_OPEN_SOUNDBOX);

        $manialink = new ManiaLink(self::MLID_WIDGET);
		$R=dechex((int)$this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_ELITEX_BG_R));
		$G=dechex((int)$this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_ELITEX_BG_G));
		$B=dechex((int)$this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_ELITEX_BG_B));
		$RGB=(string)$R.(string)$G.(string)$B;
		$A=$this->maniaControl->getSettingManager()->getSettingValue($this, self::SETTING_ELITEX_BG_A);
		$A=$A/100;
		$x=141;
		$y=10;

		//background
		$quad = new Quad();
		$manialink->addChild($quad);
		$quad->setSize(45,50);
		$quad->setPosition($x,$y);
		$quad->setBgColor($RGB);
		$quad->setZ(2);
		$quad->setOpacity($A);
		//$quad->setAction(self::ACTION_MINIMIZE_WIDGET);
		//Title
		$quad = new Quad();
		$manialink->addChild($quad);
		$quad->setSize(45,10);
		$quad->setPosition($x,$y+30);
		$quad->setBgColor("00F");
		$quad->setZ(2);
		$quad->setOpacity(0.5);
		//logo
		$quad = new Quad();
		$manialink->addChild($quad);
		$quad->setSize(10,10);
		$quad->setPosition($x-17.5,$y+30);
		$quad->setImage($this->img['logo']);
		//$quad->setAction(self::ACTION_MINIMIZE_WIDGET);
		$quad->setZ(3);

		//Minimize


		//title
		$label = new Label_Text();
		$manialink->addChild($label);
		$label->setPosition($x-17.5+12,$y+28);
		$label->setZ(3);
		$label->setText('$o$fffRound Info');
		$label->setTextSize(1);
		$label->setHAlign(Control::LEFT);



		//Attacker Title
		$label = new Label_Text();
		$manialink->addChild($label);
		$label->setPosition($x,$y+20);
		$label->setZ(8);
		$label->setText('$o$ff0$sATTACKER');
		$label->setTextSize(1);
		$label->setHAlign(Control::CENTER);

		//Echelon Attacker
		$quad = new Quad();
		$manialink->addChild($quad);
		$quad->setSize(5,5);
		$quad->setPosition($x-20,$y+15);
		$quad->setImage($this->maniaControl->getManialinkManager()->getIconManager()->getIcon($this->teams['attackerECHELON']));
		$quad->setZ(5);
		//$this->maniaControl->getChat()->sendChat($this->teams['attackerECHELON'], null, false);
		//$quad->setOpacity(0.7);

		//Attacker
		$label = new Label_Text();
		$manialink->addChild($label);
		$label->setPosition($x,$y+15);
		$label->setZ(8);
		$label->setText('$o$f00'.$this->teams['attacker']);
		$label->setTextSize(1);
		$label->setHAlign(Control::CENTER);


		//Defenders Title
		$label = new Label_Text();
		$manialink->addChild($label);
		$label->setPosition($x,$y+5);
		$label->setZ(8);
		$label->setText('$o$ff0$sDEFENDERS');
		$label->setTextSize(1);
		$label->setHAlign(Control::CENTER);


		if($this->teams['defender'][0]["status"]=="dead") {
		//RIP player 1
			$quad = new Quad();
			$manialink->addChild($quad);
			$quad->setSize(8,8);
			$quad->setPosition($x-18.5,$y-2);
			$quad->setImage($this->maniaControl->getManialinkManager()->getIconManager()->getIcon(self::ELITEX_IMG_RIP));
			$quad->setZ(8);
			//$quad->setOpacity(0.7);
		}
		elseif($this->teams['defender'][0]["status"]!=="empty")
		{
		//Echelon player 1
			$quad = new Quad();
			$manialink->addChild($quad);
			$quad->setSize(5,5);
			$quad->setPosition($x-20,$y-1.5);
			$quad->setImage($this->maniaControl->getManialinkManager()->getIconManager()->getIcon($this->teams['defender'][0]["echelon"]));
			//$this->maniaControl->getChat()->sendChat("defender 1 echelon : ".$this->teams['defender'][0]["echelon"], null, false);
			$quad->setZ(8);
			//$quad->setOpacity(0.7);
		}
		if($this->teams['defender'][1]["status"]=="dead") {
			//RIP player 2
			$quad = new Quad();
			$manialink->addChild($quad);
			$quad->setSize(8,8);
			$quad->setPosition($x-18.5,$y-10);
			$quad->setImage($this->maniaControl->getManialinkManager()->getIconManager()->getIcon(self::ELITEX_IMG_RIP));
			$quad->setZ(8);
			//$quad->setOpacity(0.7);
		}
		elseif($this->teams['defender'][1]["status"]!=="empty")
		{
		//Echelon player 2
			$quad = new Quad();
			$manialink->addChild($quad);
			$quad->setSize(5,5);
			$quad->setPosition($x-20,$y-9.5);
			$quad->setImage($this->maniaControl->getManialinkManager()->getIconManager()->getIcon($this->teams['defender'][1]["echelon"]));
			$quad->setZ(8);
			//$quad->setOpacity(0.7);
		}
		if($this->teams['defender'][2]["status"]=="dead") {
			//RIP player 3
			$quad = new Quad();
			$manialink->addChild($quad);
			$quad->setSize(8,8);
			$quad->setPosition($x-18.5,$y-18);
			$quad->setImage($this->maniaControl->getManialinkManager()->getIconManager()->getIcon(self::ELITEX_IMG_RIP));
			$quad->setZ(8);
			//$quad->setOpacity(0.7);
		}
		elseif($this->teams['defender'][2]["status"]!=="empty")
		{
		//Echelon player 3
			$quad = new Quad();
			$manialink->addChild($quad);
			$quad->setSize(5,5);
			$quad->setPosition($x-20,$y-17.5);
			$quad->setImage($this->maniaControl->getManialinkManager()->getIconManager()->getIcon($this->teams['defender'][2]["echelon"]));
			$quad->setZ(8);
			//$quad->setOpacity(0.7);
		}


		//Defender 1
		$name="";
		$label = new Label_Text();
		$manialink->addChild($label);
		$label->setPosition($x-12,$y-2);
		$label->setZ(8);
		switch ($this->teams['defender'][0]["status"])
		{
			case "left":
			$name = $this->teams['defender'][0]["login"];

			$label->setText('$999(left)$z$999$n '.$name);
			break;

			case "afk":
			$player = $this->maniaControl->getPlayerManager()->getPlayer($this->teams['defender'][0]["login"]);
			$name = $player->nickname;

			$label->setText('$999(afk)$z$999$n '.$name);
			break;

			case "dead":
			$player = $this->maniaControl->getPlayerManager()->getPlayer($this->teams['defender'][0]["login"]);
			$name = $player->nickname;

			$label->setText('$999$n '.$name);
			$label->setPosition($x-7,$y-2);
			break;

			case "empty":
			$label->setText(' ');
			break;

			case "playing":
			$player = $this->maniaControl->getPlayerManager()->getPlayer($this->teams['defender'][0]["login"]);
			$name = $player->nickname;
			$label->setText('$f00'.$name);
			break;
		}
		$label->setTextSize(1);
		$label->setHAlign(Control::LEFT);

		//Defender 2
		$name="";
		$label = new Label_Text();
		$manialink->addChild($label);
		$label->setPosition($x-12,$y-10);
		$label->setZ(8);
		switch ($this->teams['defender'][1]["status"])
		{
			case "left":
			$player = $this->maniaControl->getPlayerManager()->getPlayer($this->teams['defender'][1]["login"]);
			$name = $player->nickname;

			$label->setText('$999(left)$z$999$n '.$name);
			break;

			case "afk":
			$player = $this->maniaControl->getPlayerManager()->getPlayer($this->teams['defender'][1]["login"]);
			$name = $player->nickname;

			$label->setText('$999(afk)$z$999$n '.$name);
			break;

			case "dead":
			$player = $this->maniaControl->getPlayerManager()->getPlayer($this->teams['defender'][1]["login"]);
			$name = $player->nickname;

			$label->setText('$999$n '.$name);
			$label->setPosition($x-7,$y-10);
			break;

			case "empty":
			$label->setText(' ');
			break;

			case "playing":
			$player = $this->maniaControl->getPlayerManager()->getPlayer($this->teams['defender'][1]["login"]);
			$name = $player->nickname;
			$label->setText('$f00'.$name);
			break;
		}
		$label->setTextSize(1);
		$label->setHAlign(Control::LEFT);

		//Defender 3
		$name="";
		$label = new Label_Text();
		$manialink->addChild($label);
		$label->setPosition($x-12,$y-18);
		$label->setZ(2);
		switch ($this->teams['defender'][2]["status"])
		{
			case "left":
			$player = $this->maniaControl->getPlayerManager()->getPlayer($this->teams['defender'][2]["login"]);
			$name = $player->nickname;

			$label->setText('$999(left)$z$n$999 '.$name);
			break;

			case "afk":
			$player = $this->maniaControl->getPlayerManager()->getPlayer($this->teams['defender'][2]["login"]);
			$name = $player->nickname;

			$label->setText('$999(afk)$z$n$999 '.$name);
			break;

			case "dead":
			$player = $this->maniaControl->getPlayerManager()->getPlayer($this->teams['defender'][2]["login"]);
			$name = $player->nickname;

			$label->setText('$999$n '.$name);
			$label->setPosition($x-7,$y-18);
			break;

			case "empty":
			$label->setText(' ');
			break;

			case "playing":
			$player = $this->maniaControl->getPlayerManager()->getPlayer($this->teams['defender'][2]["login"]);
			$name = $player->nickname;
			$label->setText('$f00'.$name);
			break;
		}
		$label->setTextSize(1);
		$label->setHAlign(Control::LEFT);


		if(!$to)
		{
			$this->maniaControl->getManialinkManager()->sendManialink($manialink);
		}
		else
		{
			$player = $this->maniaControl->getPlayerManager()->getPlayer($to);
			$this->maniaControl->getManialinkManager()->sendManialink($manialink,$player);

		}

	}


    public function hide() {
        	$emptyManialink = new ManiaLink(self::MLID_WIDGET);
			$emptyManialink->removeChildren();
        	$this->maniaControl->getManialinkManager()->sendManialink($emptyManialink);

    }

    public function show() {
			$this->widgetChoice();
    }


	public function handleCallbackEliteStartTurn(OnEliteStartTurnStructure $structure) {

		$nrOfDefenders = count($structure->getDefenders());
		//$this->maniaControl->getChat()->sendChat("nr of defender : ".$nrOfDefenders, null, false);
		for ($t = 0; $t <= 2; $t++) {
			$this->teams['defender'][$t]["status"]="empty";
		}
		$t=0;
		foreach ($structure->getDefenders() as $defender) {
			//$this->maniaControl->getChat()->sendChat($defender->login . " playing..", null, false);
			$this->teams['defender'][$t]["login"]=$defender->login;
			$this->teams['defender'][$t]["status"]="playing";
			$this->teams['defender'][$t]["echelon"]=$this->getEchelon($defender->ladderScore);
			$t++;
		}
		//$this->maniaControl->getChat()->sendChat("ladder attacker: ".$structure->getAttacker()->ladderScore, null, false);
		$this->teams['attacker']=$structure->getAttacker()->nickname;
		$this->teams['attackerECHELON']=$this->getEchelon($structure->getAttacker()->ladderScore);

		$this->widgetChoice();
		$this->playing=true;
	}

	public function handleCallbackEliteEndTurn($structure) {
		$this->playing=false;
	}

	public function handleCallbackEndMap($callback) {
		$this->hide();
		$this->playing=false;
	}

	public function handleCallbackBeginWarmup() {
		if($this->playing==true) {
			$this->hide();
			$this->playing = false;
		}
	}

	public function handleCallbackOnArmorEmpty(OnArmorEmptyStructure $structure) {
		if($this->playing==false) return;
		$victim = $structure->getVictim()->login;
		for ( $t = 0; $t < 3; $t++ ) {
			if ($this->teams['defender'][$t]["status"]=="playing") {
				if($victim==$this->teams['defender'][$t]["login"]) {
					$this->teams['defender'][$t]["status"]="dead";
					$this->widgetChoice();
				}
			}
		}
	}



    public function unload() {
        $this->hide();
        $this->maniaControl->getCallbackManager()->unregisterCallbackListener($this);
        unset($this->maniaControl);
    }

     /**
     * Get plugin id
     *
     * @return int
     */
    public static function getId() {
        return self::PLUGIN_ID;
    }

    /**
     * Get Plugin Name
     *
     * @return string
     */
    public static function getName() {
        return self::PLUGIN_NAME;
    }

    /**
     * Get Plugin Version
     *
     * @return float,,
     */
    public static function getVersion() {
        return self::PLUGIN_VERSION;
    }

    /**
     * Get Plugin Author
     *
     * @return string
     */
    public static function getAuthor() {
        return self::PLUGIN_AUTHOR;
    }

    /**
     * Get Plugin Description
     *
     * @return string
     */
    public static function getDescription() {
        return 'Displays round start info.';
    }
}
